/*
 * Copyright 2015 - 2023 gary@drinkingtea.net
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */

#pragma once

#include "array.hpp"
#include "bstring.hpp"
#include "random.hpp"
#include "strops.hpp"
#include "trace.hpp"

namespace ox {

namespace detail {
constexpr ox::BString<2> toHex(uint8_t v) noexcept {
	constexpr ox::Array<char, 16> valMap {
		'0',
		'1',
		'2',
		'3',
		'4',
		'5',
		'6',
		'7',
		'8',
		'9',
		'a',
		'b',
		'c',
		'd',
		'e',
		'f',
	};
	ox::Array<char, 3> out;
	out[0] = valMap[static_cast<unsigned>((v & 0xf0) / 0xf)];
	out[1] = valMap[static_cast<unsigned>(v & 0x0f)];
	out[2] = 0;
	return out.data();
}
}

class UUID {
	private:
		static bool s_seeded;
		static Random s_rand;
		ox::Array<uint8_t, 16> m_value;

	public:
		static void seed(const RandomSeed &seed) noexcept;

		static ox::Result<UUID> generate() noexcept;

		[[nodiscard]]
		constexpr ox::BString<36> toString() const noexcept {
			ox::BString<36> out;
			auto i = 0u;
			constexpr auto printChars = [](
					ox::BString<36> *out, const Array<uint8_t, 16> &value, std::size_t cnt, unsigned i) {
				for (; i < cnt; ++i) {
					const auto v = value[i];
					const auto h = detail::toHex(v);
					oxIgnoreError(out->append(h.c_str(), h.len()));
				}
				return i;
			};
			i = printChars(&out, m_value, 4, i);
			out += "-";
			i = printChars(&out, m_value, 2, i);
			out += "-";
			i = printChars(&out, m_value, 2, i);
			out += "-";
			i = printChars(&out, m_value, 2, i);
			out += "-";
			i = printChars(&out, m_value, 6, i);
			return out;
		}
};

}
